import 'dart:async';
import 'dart:developer';

import 'package:flutter/services.dart';

typedef UserBalanceChanged = void Function(int balance);
typedef InitializationSuccess = void Function(int balance, int spent, int pending);
typedef InitializationFailed = void Function();
typedef VideoAborted = void Function();
typedef VideoCompleted = void Function(int amount);
typedef VideoNofill = void Function();
typedef VideoFinished = void Function();
typedef VideoWillBeShown = void Function();

class CsAyetstudiosAndroid {
  final MethodChannel _channel = const MethodChannel('cs_ayetstudios_android');
  static final CsAyetstudiosAndroid _instance = CsAyetstudiosAndroid._internal();
  factory CsAyetstudiosAndroid() => _instance;
  CsAyetstudiosAndroid._internal() {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  late UserBalanceChanged _userBalanceChanged;
  late InitializationSuccess _initializationSuccess;
  late InitializationFailed _initializationFailed;

  late VideoAborted _videoAborted;
  late VideoCompleted _videoCompleted;
  late VideoNofill _videoNofill;
  late VideoFinished _videoFinished;
  late VideoWillBeShown _videoWillBeShown;

  Future<void> initSDK({required String uid}) async {
    return _channel.invokeMethod("init", <String, dynamic>{
      'uid': uid,
    });
  }

  /// SDK是否初始化完成
  Future<bool?> isInitialized() {
    return _channel.invokeMethod<bool>("isInitialized");
  }

  /// 获取当前可用积分
  Future<int?> getAvailableBalance() {
    return _channel.invokeMethod<int>("getAvailableBalance");
  }

  /// 获取当前花费掉积分
  Future<int?> getSpentBalance() {
    return _channel.invokeMethod<int>("getSpentBalance");
  }

  /// 获取等待的积分
  Future<int?> getPendingBalance() {
    return _channel.invokeMethod<int>("getPendingBalance");
  }

  /// 展示积分墙
  Future<void> showOfferwallAd(String name) {
    return _channel.invokeMethod("showOfferwallAd", <String, dynamic>{
      'name': name,
    });
  }

  /// 展示视频广告
  Future<void> showVideoAd(String name) {
    return _channel.invokeMethod("showVideoAd", <String, dynamic>{
      'name': name,
    });
  }

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "userBalanceChanged":
        _userBalanceChanged(int.parse(call.arguments[0] ?? "0"));
        break;
      case "initializationFailed":
        _initializationFailed();
        break;
      case "initializationSuccess":
        _initializationSuccess(int.parse(call.arguments[0] ?? "0"), int.parse(call.arguments[1] ?? "0"), int.parse(call.arguments[2] ?? "0"));
        break;
      case "videoAborted":
        _videoAborted();
        break;
      case "videoCompleted":
        _videoCompleted(int.parse(call.arguments[0] ?? "0"));
        break;
      case "videoNofill":
        _videoNofill();
        break;
      case "videoFinished":
        _videoFinished();
        break;
      case "videoWillBeShown":
        _videoWillBeShown();
        break;
      default:
        log('Unknown method ${call.method} ');
    }
  }

  /// 用户积分发生变化
  void setUserBalanceChangedListener(UserBalanceChanged userBalanceChanged) => _userBalanceChanged = userBalanceChanged;

  /// SDK初始化成功
  void setInitializationSuccessListener(InitializationSuccess initializationSuccess) => _initializationSuccess = initializationSuccess;

  /// SDK初始化失败
  void setInitilizationFailedListener(InitializationFailed initializationFailed) => _initializationFailed = initializationFailed;

  /// 视频放弃 或者跳过
  void setVideoAbortedListener(VideoAborted videoAborted) => _videoAborted = videoAborted;

  /// 看完完整视频获得奖励回调
  void setVideoCompletedListener(VideoCompleted videoCompleted) => _videoCompleted = videoCompleted;

  /// 没有获得视频奖励可以展示
  void setVideoNofillListener(VideoNofill videoNofill) => _videoNofill = videoNofill;

  /// 所有视频活动完成时都会调用(视频完成，中止或其他问题，如网络速度)
  void setVideoFinishedListener(VideoFinished videoFinished) => _videoFinished = videoFinished;

  /// 择一个视频之后和视频活动开始播放它之前被调用
  void setVideoWillBeShownListener(VideoWillBeShown videoWillBeShown) => _videoWillBeShown = videoWillBeShown;
}

class CsAyetstudiosIos {
  final MethodChannel _channel = const MethodChannel('cs_ayetstudios_ios');
  static final CsAyetstudiosIos _instance = CsAyetstudiosIos._internal();
  factory CsAyetstudiosIos() => _instance;

  late UserBalanceChanged _userBalanceChanged;

  CsAyetstudiosIos._internal() {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  Future<void> initSDK({required String uid, appKey}) async {
    return _channel.invokeMethod("init", <String, dynamic>{
      'appKey': appKey,
      'uid': uid,
    });
  }

  void sdkLogEnable() async {
    return _channel.invokeMethod("sdkLogEnable");
  }

  void userAvailableBalance() async {
    return _channel.invokeMethod("userAvailableBalance");
  }

  void userPendingBalance() async {
    return _channel.invokeMethod("userPendingBalance");
  }

  Future<void> show(String placement) {
    return _channel.invokeMethod("show", <String, dynamic>{
      'placement': placement,
    });
  }

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "userBalanceChanged":
        _userBalanceChanged(call.arguments);
        break;
      default:
        print('Unknown method ${call.method} ');
    }
  }

  void setUserBalanceChanged(UserBalanceChanged userBalanceChanged) => _userBalanceChanged = userBalanceChanged;
}
